package dsexamples.misc;

import java.awt.*;
import java.io.*;
import java.awt.image.*;
import java.awt.Component;
import java.awt.event.*;

public class Meter extends Canvas {

  private int m_FillColorRGB;           // Color of the Meter fill in RGB bits.

  private Color m_cBackground = Color.lightGray;
  private Color m_cFrame = Color.black;

  private int m_XCenter;			// X coordinate of the center of the thermometer.
  private int m_YCenter;			// Y coordinate of the center of the thermometer.
  private int m_barWidth;			// Width of the thermometer bar.
  private int m_barHeight;			// Height of the thermometer bar

  private float m_minValue;			        // Minimum value of the thermometer.
  private float m_maxValue;		          // Maximum value of the thermometer.
  private String m_sLabel;
  private String m_sTitle;
  private boolean m_bDrawBulb;

  private Image m_backgroundPicture;		// Background image.
  private Image m_scaledBackgroundPicture;   // Cached scaled background picture
  private Image m_offScreenImage;
  private Graphics m_offScreenGraphics = null;
  private boolean m_bPictLoaded = false;		// Boolean indicating whether the Boolean has been drawn.
  private boolean m_bPainted = false;

  private float m_value;

  /***** Constructor *****/
  Meter (int FillColorRGB,
                float minValue,
         	      float maxValue,
                String sLabel,
                String sTitle,
         	      Image backgroundPicture)
  {
    m_FillColorRGB = FillColorRGB;
    m_minValue = minValue;
    m_maxValue = maxValue;
    m_backgroundPicture = backgroundPicture;
    m_sLabel = sLabel;
    m_sTitle = sTitle;
    m_bDrawBulb = true;
    enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
  } // End Constructor.

  public void setDrawBulb(boolean bDrawBulb) { m_bDrawBulb = bDrawBulb; }
  public boolean getDrawBulb() { return m_bDrawBulb; }

  /***** Methods *****/

  public void updateValue(float value)
  { // Method to update the state of the thermometer.
    m_value = value;
    if (!m_bPainted) {
      Resized();
    }
    if (m_bPainted) {
      PaintImage();
    }
    repaint();
  }

  private void Resized() {
    Rectangle rbounds = this.getBounds();

  if (rbounds.height != 0 && rbounds.width != 0) {
      m_bPictLoaded = false;
      m_scaledBackgroundPicture = m_backgroundPicture.getScaledInstance(rbounds.width,
                                          rbounds.height, Image.SCALE_FAST);
      m_XCenter = rbounds.width / 2;
      m_YCenter = rbounds.height / 2;
      m_barWidth = rbounds.width / 20;
      m_barHeight = (int)(Math.max(rbounds.height * 0.4, rbounds.height - 70));
      m_offScreenImage = createImage(rbounds.width, rbounds.height);
      m_offScreenGraphics = m_offScreenImage.getGraphics();

      prepareImage(m_scaledBackgroundPicture, -1, -1, this);
      m_bPainted = true;
    }
  }

  protected void processComponentEvent(ComponentEvent e) {
    switch (e.getID()) {
      case ComponentEvent.COMPONENT_RESIZED:
        // if we resize, go ahead and recalculate image
        Resized();
        break;
      default: break;
    };
    super.processComponentEvent(e);
  }

  public void update (Graphics g)
  { paint(g);
  }

  // Paint to off-screen graphics image
  private void PaintImage()
  {
    Rectangle rbounds = this.getBounds();
    Graphics g = m_offScreenGraphics;
    // if resized, recalculation has already been done

    g.drawImage(m_scaledBackgroundPicture, 0, 0, null);

    // Calculate vertical extent of filler in bar, given value, in fractional pixels.
    float valueHeight = (float)m_barHeight * ((m_value - m_minValue)/(m_maxValue - m_minValue));

    // Instantiate object to hold color of fill in bar.
    Color barColor = new Color(m_FillColorRGB);

    // Calculate X coordinate of top left of bar.
    int XTopCorner = m_XCenter - (int)(m_barWidth/2);

    // Calculate Y coordinate of top left of bar.
    int YTopCorner = m_YCenter - (int)(m_barHeight/2);

    // Draw thermometer bar outline.
    g.setColor(Color.black);
    g.drawRect(XTopCorner, YTopCorner, m_barWidth, m_barHeight);

    // Draw real-time display.
    Font font1 = new Font("Arial", Font.PLAIN, 12);
    g.setFont(font1);

    // Draw Title
    FontMetrics fm = g.getFontMetrics();
    int xTitleStart = rbounds.width/2 - fm.stringWidth(m_sTitle)/2;
    g.drawString(m_sTitle, xTitleStart, (int)(fm.getHeight() * 1.5));

    String sValue = Float.toString(m_value);
    int maxl = Math.min(sValue.length(), 4);
    g.drawString((sValue.substring(0,maxl)) + " " + m_sLabel,
				   XTopCorner + m_barWidth + 5,
				   YTopCorner + 15);

    // Draw upper number marker.
    Font font2 = new Font("Arial", Font.PLAIN, 10);
    g.setFont(font2);
    g.drawString(Float.toString(m_maxValue),
				   XTopCorner - 7*(int)(Float.toString(m_maxValue).length()),
				   YTopCorner + 5);

    // Draw lower number marker.
    g.drawString(Float.toString(m_minValue),
				   XTopCorner - (int)(Float.toString(m_minValue).length())*7,
				   YTopCorner + m_barHeight);

    // Fill thermometer bar with background color.
    g.setColor(Color.gray);
    g.fillRect(XTopCorner + 1, YTopCorner + 1,
				 m_barWidth - 1, m_barHeight - 1);

    if (m_bDrawBulb) {

      // Draw thermometer bulb circular frame.
      g.setColor(Color.black);
      g.drawOval((int)(m_XCenter - 0.75*m_barWidth),
				 (int)(YTopCorner + m_barHeight - (0.191)*m_barWidth),
				 (int)(1.5*m_barWidth),
				 (int)(1.5*m_barWidth));

      // Draw thermometer bulb fill color.
      g.setColor(barColor);
      g.fillOval((int)(m_XCenter - 0.75*m_barWidth + 1),
				 (int)(YTopCorner + m_barHeight - (0.191)*m_barWidth),
				 (int)(1.5*m_barWidth),
				 (int)(1.5*m_barWidth));
    }

    // Draw fill on thermometer bar, based on data value.
    g.setColor(barColor);
    g.fillRect(XTopCorner + 1,
				 YTopCorner + m_barHeight - (int)valueHeight + 1,
				 m_barWidth - 1,
				 (int)valueHeight - 1);
  }

  public synchronized boolean imageUpdate(Image image, int infoFlags,
        int x, int y, int width, int height)
  {
    if ((infoFlags & ImageObserver.ALLBITS) != 0) {
      m_bPictLoaded = true;
      PaintImage();
      repaint();
      return false;
    }
    else {
      return true;
    }
  }

  public void paint(Graphics g)
  {
    // draw offscreen image now
    if (m_bPictLoaded)
      g.drawImage(m_offScreenImage, 0, 0, this);
  } // End paint(g)

} // End Meter.

